@@ -26,6 +26,7 @@ pub(crate) fn help(
     ty_attrs: &TypeAttrs,
     fields: &[StructField<'_>],
     subcommand: Option<&StructField<'_>>,
+    help_triggers: &[String],
 ) -> TokenStream {
     let mut format_lit = "Usage: {command_name}".to_string();
 
@@ -83,8 +84,12 @@ pub(crate) fn help(
     for option in options {
         option_description(errors, &mut format_lit, option);
     }
-    // Also include "help"
-    option_description_format(&mut format_lit, None, "--help", "display usage information");
+    option_description_format(
+        &mut format_lit,
+        None,
+        &help_triggers.join(", "),
+        "display usage information",
+    );
 
     let subcommand_calculation;
     let subcommand_format_arg;
@@ -157,7 +162,7 @@ fn positional_usage(out: &mut String, field: &StructField<'_>) {
     }
     let name = field.positional_arg_name();
     out.push_str(&name);
-    if field.optionality == Optionality::Repeating {
+    if matches!(field.optionality, Optionality::Repeating | Optionality::DefaultedRepeating(_)) {
         out.push_str("...");
     }
     if field.attrs.greedy.is_none() {
@@ -194,7 +199,10 @@ fn option_usage(out: &mut String, field: &StructField<'_>) {
             } else {
                 out.push_str(long_name.trim_start_matches("--"));
             }
-            if field.optionality == Optionality::Repeating {
+            if matches!(
+                field.optionality,
+                Optionality::Repeating | Optionality::DefaultedRepeating(_)
+            ) {
                 out.push_str("...");
             }
             out.push('>');
