@@ -3,6 +3,8 @@
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
+use syn::ext::IdentExt as _;
+
 /// Implementation of the `FromArgs` and `argh(...)` derive attributes.
 ///
 /// For more thorough documentation, see the `argh` crate itself.
@@ -67,6 +69,7 @@ enum Optionality {
     Defaulted(TokenStream),
     Optional,
     Repeating,
+    DefaultedRepeating(TokenStream),
 }
 
 impl PartialEq<Optionality> for Optionality {
@@ -159,8 +162,14 @@ impl<'a> StructField<'a> {
                             tree
                         })
                         .collect();
-                    optionality = Optionality::Defaulted(tokens);
-                    ty_without_wrapper = &field.ty;
+                    let inner = if let Some(x) = ty_inner(&["Vec"], &field.ty) {
+                        optionality = Optionality::DefaultedRepeating(tokens);
+                        x
+                    } else {
+                        optionality = Optionality::Defaulted(tokens);
+                        &field.ty
+                    };
+                    ty_without_wrapper = inner;
                 } else {
                     let mut inner = None;
                     optionality = if let Some(x) = ty_inner(&["Option"], &field.ty) {
@@ -188,7 +197,7 @@ impl<'a> StructField<'a> {
         let long_name = match kind {
             FieldKind::Switch | FieldKind::Option => {
                 let long_name = attrs.long.as_ref().map(syn::LitStr::value).unwrap_or_else(|| {
-                    let kebab_name = to_kebab_case(&name.to_string());
+                    let kebab_name = to_kebab_case(&name.unraw().to_string());
                     check_long_name(errors, name, &kebab_name);
                     kebab_name
                 });
@@ -363,9 +372,15 @@ fn impl_from_args_struct_from_args<'a>(
         quote_spanned! { impl_span => None }
     };
 
-    // Identifier referring to a value containing the name of the current command as an `&[&str]`.
-    let cmd_name_str_array_ident = syn::Ident::new("__cmd_name", impl_span);
-    let help = help::help(errors, cmd_name_str_array_ident, type_attrs, fields, subcommand);
+    let help_triggers = get_help_triggers(type_attrs);
+
+    let help = if cfg!(feature = "help") {
+        // Identifier referring to a value containing the name of the current command as an `&[&str]`.
+        let cmd_name_str_array_ident = syn::Ident::new("__cmd_name", impl_span);
+        help::help(errors, cmd_name_str_array_ident, type_attrs, fields, subcommand, &help_triggers)
+    } else {
+        quote! { String::new() }
+    };
 
     let method_impl = quote_spanned! { impl_span =>
         fn from_args(__cmd_name: &[&str], __args: &[&str])
@@ -381,6 +396,7 @@ fn impl_from_args_struct_from_args<'a>(
                 argh::ParseStructOptions {
                     arg_to_slot: &[ #( #flag_str_to_output_table_map ,)* ],
                     slots: &mut [ #( #flag_output_table, )* ],
+                    help_triggers: &[ #( #help_triggers ),* ],
                 },
                 argh::ParseStructPositionals {
                     positionals: &mut [
@@ -413,6 +429,29 @@ fn impl_from_args_struct_from_args<'a>(
     method_impl
 }
 
+/// get help triggers vector from type_attrs.help_triggers as a [`Vec<String>`]
+///
+/// Defaults to vec!["--help", "help"] if type_attrs.help_triggers is None
+fn get_help_triggers(type_attrs: &TypeAttrs) -> Vec<String> {
+    let help_triggers = type_attrs.help_triggers.as_ref().map_or_else(
+        || vec!["--help".to_owned(), "help".to_owned()],
+        |s| {
+            s.iter()
+                .filter_map(|s| {
+                    let trigger = s.value();
+                    let trigger_trimmed = trigger.trim().to_owned();
+                    if trigger_trimmed.is_empty() {
+                        None
+                    } else {
+                        Some(trigger_trimmed)
+                    }
+                })
+                .collect::<Vec<_>>()
+        },
+    );
+    help_triggers
+}
+
 fn impl_from_args_struct_redact_arg_values<'a>(
     errors: &Errors,
     type_attrs: &TypeAttrs,
@@ -483,9 +522,15 @@ fn impl_from_args_struct_redact_arg_values<'a>(
         quote! { "no subcommand name" }
     };
 
-    // Identifier referring to a value containing the name of the current command as an `&[&str]`.
-    let cmd_name_str_array_ident = syn::Ident::new("__cmd_name", impl_span);
-    let help = help::help(errors, cmd_name_str_array_ident, type_attrs, fields, subcommand);
+    let help_triggers = get_help_triggers(type_attrs);
+
+    let help = if cfg!(feature = "help") {
+        // Identifier referring to a value containing the name of the current command as an `&[&str]`.
+        let cmd_name_str_array_ident = syn::Ident::new("__cmd_name", impl_span);
+        help::help(errors, cmd_name_str_array_ident, type_attrs, fields, subcommand, &help_triggers)
+    } else {
+        quote! { String::new() }
+    };
 
     let method_impl = quote_spanned! { impl_span =>
         fn redact_arg_values(__cmd_name: &[&str], __args: &[&str]) -> std::result::Result<Vec<String>, argh::EarlyExit> {
@@ -497,6 +542,7 @@ fn impl_from_args_struct_redact_arg_values<'a>(
                 argh::ParseStructOptions {
                     arg_to_slot: &[ #( #flag_str_to_output_table_map ,)* ],
                     slots: &mut [ #( #flag_output_table, )* ],
+                    help_triggers: &[ #( #help_triggers ),* ],
                 },
                 argh::ParseStructPositionals {
                     positionals: &mut [
@@ -597,8 +643,11 @@ fn top_or_sub_cmd_impl(
     type_attrs: &TypeAttrs,
     generic_args: &syn::Generics,
 ) -> TokenStream {
-    let description =
-        help::require_description(errors, name.span(), &type_attrs.description, "type");
+    let description = if cfg!(feature = "help") {
+        help::require_description(errors, name.span(), &type_attrs.description, "type")
+    } else {
+        String::new()
+    };
     let (impl_generics, ty_generics, where_clause) = generic_args.split_for_impl();
     if type_attrs.is_subcommand.is_none() {
         // Not a subcommand
@@ -642,6 +691,9 @@ fn declare_local_storage_for_from_args_fields<'a>(
             Optionality::None | Optionality::Defaulted(_) => {
                 quote! { std::option::Option<#field_type> }
             }
+            Optionality::DefaultedRepeating(_) => {
+                quote! { std::option::Option<std::vec::Vec<#field_type>> }
+            }
         };
 
         match field.kind {
@@ -687,7 +739,7 @@ fn unwrap_from_args_fields<'a>(
                 Optionality::Optional | Optionality::Repeating => {
                     quote! { #field_name: #field_name.slot }
                 }
-                Optionality::Defaulted(tokens) => {
+                Optionality::Defaulted(tokens) | Optionality::DefaultedRepeating(tokens) => {
                     quote! {
                         #field_name: #field_name.slot.unwrap_or_else(|| #tokens)
                     }
@@ -697,7 +749,7 @@ fn unwrap_from_args_fields<'a>(
             FieldKind::SubCommand => match field.optionality {
                 Optionality::None => quote! { #field_name: #field_name.unwrap() },
                 Optionality::Optional | Optionality::Repeating => field_name.into_token_stream(),
-                Optionality::Defaulted(_) => unreachable!(),
+                Optionality::Defaulted(_) | Optionality::DefaultedRepeating(_) => unreachable!(),
             },
         }
     })
@@ -727,6 +779,9 @@ fn declare_local_storage_for_redacted_fields<'a>(
                     Optionality::Repeating => {
                         quote! { std::vec::Vec<String> }
                     }
+                    Optionality::DefaultedRepeating(_) => {
+                        quote! { std::option::Option<std::vec::Vec<String>> }
+                    }
                     Optionality::None | Optionality::Optional | Optionality::Defaulted(_) => {
                         quote! { std::option::Option<String> }
                     }
@@ -745,6 +800,9 @@ fn declare_local_storage_for_redacted_fields<'a>(
                     Optionality::Repeating => {
                         quote! { std::vec::Vec<String> }
                     }
+                    Optionality::DefaultedRepeating(_) => {
+                        quote! { std::option::Option<std::vec::Vec<String>> }
+                    }
                     Optionality::None | Optionality::Optional | Optionality::Defaulted(_) => {
                         quote! { std::option::Option<String> }
                     }
@@ -787,6 +845,13 @@ fn unwrap_redacted_fields<'a>(
                         __redacted.extend(#field_name.slot.into_iter());
                     }
                 }
+                Optionality::DefaultedRepeating(_) => {
+                    quote! {
+                        if let Some(__field_name) = #field_name.slot {
+                            __redacted.extend(__field_name.into_iter());
+                        }
+                    }
+                }
                 Optionality::None | Optionality::Optional | Optionality::Defaulted(_) => {
                     quote! {
                         if let Some(__field_name) = #field_name.slot {
